;REMOTE SCRIPT START
g_.Spline :=
(
"
sampler s0 : register(s0);
float4 Spline(float2 tex : TEXCOORD0) : COLOR0
{
	// adapted from:
	// https://gist.github.com/TheRealMJP/c83b8c0f46b63f3a88a5986f4fa982b1
	
	// We're going to sample a a 4x4 grid of texels surrounding the target UV coordinate. We'll do this by rounding
    // down the sample location to get the exact center of our starting texel. The starting texel will be at
	// location [1, 1] in the grid, where [0, 0] is the top left corner.
	// Note: to realy sample the 4x4 grid, texture filtering must be enabled, otherwise we really only sample
	// one texel
	
	float2 texSize   = float2(ww., hh.);	
    float2 samplePos = tex * texSize;
    float2 texPos1   = floor(samplePos - 0.5f) + 0.5f;
	
	// Compute the fractional offset from our starting texel to our original sample location, which we'll
	// feed into the Catmull-Rom spline function to get our filter weights.
	float2 f         = samplePos - texPos1;
		
    float2 w0 = f * (-0.5f + f * (1.0f - 0.5f * f));
    float2 w1 = 1.0f + f * f * (-2.5f + 1.5f * f);
    float2 w2 = f * (0.5f + f * (2.0f - 1.5f * f));
    float2 w3 = f * f * (-0.5f + 0.5f * f);

    float2 w12 = w1 + w2;
    float2 offset12 = w2 / (w1 + w2);

    float2 texPos0 = texPos1 - 1;
    float2 texPos3 = texPos1 + 2;
    float2 texPos12 = texPos1 + offset12;

    texPos0 /= texSize;
    texPos3 /= texSize;
    texPos12 /= texSize;	
				
    float4 result = float4(0., 0., 0., 0.);
	result += tex2D(s0, float2(texPos0.x, texPos0.y)) * w0.x * w0.y;
    result += tex2D(s0, float2(texPos12.x, texPos0.y)) * w12.x * w0.y;
    result += tex2D(s0, float2(texPos3.x, texPos0.y)) * w3.x * w0.y;

    result += tex2D(s0, float2(texPos0.x, texPos12.y)) * w0.x * w12.y;
    result += tex2D(s0, float2(texPos12.x, texPos12.y)) * w12.x * w12.y;
    result += tex2D(s0, float2(texPos3.x, texPos12.y)) * w3.x * w12.y;

    result += tex2D(s0, float2(texPos0.x, texPos3.y)) * w0.x * w3.y;
    result += tex2D(s0, float2(texPos12.x, texPos3.y)) * w12.x * w3.y;
    result += tex2D(s0, float2(texPos3.x, texPos3.y)) * w3.x * w3.y;
	
	float3 black    = float3(0., 0., 0.);
	if ( all(result.rgb == black) )
		discard;
    return result;
}
"
)

g_.shogo_shader_code :=
(
"
sampler s0 : register(s0);
float4 main(float2 tex : TEXCOORD0, float4 diff : COLOR0) : COLOR
{
	
	// compute floating point stride for texture
	float step_x = 1./ww.;
	float step_y = 1./hh.;

	// compute texel array coordinates
	int target_x = tex.x * ww.;
	int target_y = tex.y * hh.;

	// round to values, such that they're multiples of step_x and step_y
	// and add a half pixel offset
	float2 texCoordNew;
	texCoordNew.x = (target_x * step_x) + step_x/2.;
	texCoordNew.y = (target_y * step_y) + step_y/2.;	
	
	float4 c0       = tex2D(s0, texCoordNew);
	float3 black    = float3(0., 0., 0.);
	if ( all(c0.rgb == black) )
		discard;	
	return c0*diff;			
}
"
)

;g_.cfg.shogo_filter := True
if g_.cfg.shogo_filter
	  HDHOOKS_DATA.BltPrimary  := RegisterCallback("RenderTarget4_blt", "F")
	, logerr(IDirectDrawSurface4.UnHook("flip"))
	, logerr(IDirectDrawSurface4.Hook("flip", "ShogoFlip"))

Alt_CreateSurface(pIDirectDraw4, pSurfaceDesc, ppSurface, pIUnknown)
{
	g_.DDSDesc2[] := pSurfaceDesc
	if ! (g_.DDSDesc2.ddsCaps.dwCaps & DDSCAPS_PRIMARYSURFACE) and ! (g_.DDSDesc2.dwFlags & DDSD_PIXELFORMAT) 
		setPixelFormat(g_.DDSDesc2,  "RG6B")
		,g_.DDSDesc2.dwFlags        |= DDSD_PIXELFORMAT
		,g_.DDSDesc2.ddsCaps.dwCaps &= ~ DDSCAPS_SYSTEMMEMORY		
		
    r := IDirectDraw4_CreateSurface(pIDirectDraw4, pSurfaceDesc, ppSurface, pIUnknown)
	
		
	if (g_.DDSDesc2.ddsCaps.dwCaps & DDSCAPS_PRIMARYSURFACE) 
	{
		ww := g_.requested_res.w
		hh := g_.requested_res.h
		code := g_.shogo_shader_code
		stringreplace, code, code, ww, %ww%, 1
		stringreplace, code, code, hh, %hh%, 1
		g_.shogo_shader := d3D9_DDI_CreatePixelShader(code, "main")
		isobject(g_.shogo_shader) ?: logerr(g_.shogo_shader)
		
		code := g_.Spline
		stringreplace, code, code, ww, %ww%, 1
		stringreplace, code, code, hh, %hh%, 1
		g_.Spline_shader := d3D9_DDI_CreatePixelShader(code, "Spline")
		isobject(g_.Spline_shader) ?: logerr(g_.Spline_shader)	

		(g_.cfg.shogo_filter) ? g_.proxies.shogo_dev      := new SurrogateSurface4(,,True)		
	}	
	return r
}

RenderTarget4_blt(p1, p2, p3, p4, p5, p6)
{
	if (GetSurfaceCaps4(p1) & DDSCAPS_3DDEVICE)
	{
		g_HD.update := True	
		if (p5 & DDBLT_KEYSRC) 	
			g_.cfg.Shogo_UseCkey := True 
		return dllcall(IDirectDrawSurface4.blt, uint, g_.proxies.shogo_dev.Surface4, uint, p2, uint, p3, uint, p4, uint, p5, uint, p6, uint)
	}return dllcall(IDirectDrawSurface4.blt, uint, p1, uint, p2, uint, p3, uint, p4, uint, p5, uint, p6, uint)				
}

Shogoflip(p1, p2, p3){
	
	if g_HD.update 
	{
		(g_.cfg.Shogo_UseCkey) ? d3D9_DDI_SetShaderOverride(g_.shogo_shader)
		(g_.cfg.Shogo_UseCkey) ? d3D9_DDI_SetShaderOverride(g_.Spline_shader)
		dllcall(IDirect3DDevice3.GetTextureStageState, uint, DEVICE3_RECT.Device3, uint, 0, uint, 16, "uint*", filter)
		dllcall(IDirect3DDevice3.SetTextureStageState, uint, DEVICE3_RECT.Device3, uint, 0, uint, 16, uint, 2)
		DEVICE3_RECT.Device3 :=  g_.Device3
		DEVICE3_RECT.Texture :=  g_.proxies.shogo_dev.Texture2				        
		dllcall(g_.Dev3_DrawRect, uint, DEVICE3_RECT[], uint, _RECT[], uint, 0, uint)
		dllcall(IDirect3DDevice3.SetTextureStageState, uint, DEVICE3_RECT.Device3, uint, 0, uint, 16, uint, filter)	
		(g_.cfg.Shogo_UseCkey) ? d3D9_DDI_RestoreShader()			
		dllcall(IDirectdrawSurface4.blt, uint, g_.proxies.shogo_dev.Surface4, uint, 0, uint, 0, uint, 0
									   , uint, DDBLT_COLORFILL, uint, DDBLTFX[], uint)								   
		g_HD.update                 := 0
		g_.surrogate.UseCkey := False			
	}
	r := dllcall(IDirectDrawSurface4.flip, uint, p1, uint, p2, uint, p3)	
	return r										  
}

/*
AltIDirectDrawSurface4_QueryInterface(p1, p2, p3)
{
	; Extra check for mipmaps, so non mipmaped texures won't be managed, avoids a little stuttering
	isTexture := ? True : False
	if ( (StringFromIID(p2) = d3d.IID_IDirect3DTexture2)  and g_.cfg.ManageTextures )
	{
		if dllcall(IDirectDrawSurface4.GetSurfacedesc, uint, p1, uint, g_manager.OriginalfDesc[]) 
		   or !(g_manager.OriginalfDesc.ddsCaps.dwCaps & DDSCAPS_MIPMAP)
		   return dllcall(IDirectDrawSurface4.QueryInterface, uint, p1, uint, p2, uint, p3)
	}
	return IDirectDrawSurface4_QueryInterface(p1, p2, p3)	
}
*/